"""
    Copyright 2006,2007 Riccardo Taormina (den3b@hotmail.com)
    
    This file is part of SmartChameleons.

    SmartChameleons is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    SmartChameleons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SmartChameleons; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

#The MODEL
from Chameleon import Chameleon
import math
from mappings import *
from RL_CON import *
from ActionGroup import ActionGroup

class Model:
    def __init__(self, parent, nChams, colorList, nCycles, worldXSize,\
                 worldYSize, mode, brain,smartColor):
        # the environment
        self.time = 0
        self.nChams = nChams*3
        self.chList = []
        self.colorList = colorList
        self.nCycles = nCycles
        self.worldXSize = worldXSize
        self.worldYSize = worldYSize
        self.parent = parent
        self.chams_x_color = [0]*3
        
        self.mode = mode
        print self.mode
        self.AI = RL_CON(brain,self,self.mode)
        self.smartColor = smartColor



    # objects
    def buildObjects(self):
        i = 0
        for color in self.colorList:
            for j in range(self.nChams/3):
                newCham = Chameleon(i+j,color,None,self.worldXSize,self.worldYSize)
                self.chList.append(newCham)
            i+=self.nChams/3
            

    # actions         
    def step(self):
        self.time += 1
        for actionType in self.actionGroupList :
            if actionType == 'chameleons':
                for action in self.chameleonsActionGroup.actions :
                    action(self)
            elif actionType == 'check_model' :
                self.checkActionGroup.action(self)
                

    def buildActions(self):

        self.chameleonsActionGroup = ActionGroup ("chameleons")

        def moveChams(self):
            random.shuffle(self.chList)
            for aCham in self.chList:
                if aCham.color == self.smartColor :
                    state = self.checkState(aCham)
                    self.AI.smartMove(aCham)
                else :
                    aCham.move()

        def handleCollisions(self):
            for cham in self.chList:
                cham.justChanged = False
            i = 0
            for cham1 in self.chList:
                if cham1.hasntJustChanged():
                    for cham2 in self.chList[i+1:self.nChams*3]:
                        if cham2.hasntJustChanged():
                            if cham1.color != cham2.color and \
                                self.distance(cham1.xPosition,cham1.yPosition,
                                        cham2.xPosition,cham2.yPosition) ==0 :
                                self.changeColors(cham1,cham2)
                i += 1

        def assignRewards(self):
            if self.AI.mode == "T" :
                for cham in self.chList:
                    if cham.color == self.smartColor:
                        self.AI.assignReward(cham)


        self.chameleonsActionGroup.actions = [moveChams, handleCollisions, assignRewards]


        self.checkActionGroup = ActionGroup ("check")
            
        def check(self):
            flag = True
            color = self.chList[0].color
            for chameleon in self.chList[1:]:
                if chameleon.color != color:
                    flag = False
                    break
            self.parent.onlyOneColor = flag

        self.checkActionGroup.action = check


        self.actionGroupList = ["chameleons", "check_model"]




    def distance(self,x1,y1,x2,y2):
        x1 = (x1+self.worldXSize) % self.worldXSize
        x2 = (x2+self.worldXSize) % self.worldXSize
        y1 = (y1+self.worldYSize) % self.worldYSize
        y2 = (y2+self.worldYSize) % self.worldYSize
        return math.sqrt((x1-x2)**2+(y1-y2)**2)


    def checkState(self,cham):
        state=[0]*25
        for otherCham in self.chList:
            if cham.color != otherCham.color :
                distance = self.distance(cham.xPosition,cham.yPosition,
                                         otherCham.xPosition,otherCham.yPosition)
                if distance <3:
                    position = [cham.xPosition - otherCham.xPosition,cham.yPosition - otherCham.yPosition]
                    state[positionToBox(position)]=1                        
        return state


    def changeColors(self,cham1,cham2):
        
        if cham1.color == self.smartColor :
            cham1.firstReward = 0
            cham1.secondReward = 0
            cham1.waitingForReward = False
            cham1.waitingForLastReward = False
            self.AI.assignReward(cham1,True)
            
            
        elif cham2.color == self.smartColor:
            cham2.firstReward = 0
            cham2.secondReward = 0
            cham2.waitingForReward = False
            cham2.waitingForLastReward = False
            self.AI.assignReward(cham2,True)

                        
        colorList = self.colorList[:]
        colorList.remove(cham1.color)
        colorList.remove(cham2.color)
        otherColor = colorList[0]
        cham1.changeColor(otherColor)
        cham2.changeColor(otherColor)

        if otherColor == self.smartColor:
            cham1.waitingForReward = False
            cham2.waitingForReward = False
        



