"""
    Copyright 2006,2007 Riccardo Taormina (den3b@hotmail.com)
    
    This file is part of SmartChameleons.

    SmartChameleons is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    SmartChameleons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SmartChameleons; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""

from mappings import *
import random


global maxRew, maxPen
maxRew = [0,1,2,3,4,5,5,5,0]
maxPen = [5,5,5,4,3,2,1,0,0]


class RL_CON:
    def __init__(self,brain,parent,mode):
        self.parent = parent
        self.brain = brain
        self.mode = mode

    def smartMove(self,chameleon):
        cham = chameleon
        if self.mode == "R":
            state=self.parent.checkState(cham)
            [positionChanges,actionIndex] = self.followPolicy(state)
            cham.smartMove(positionChanges)
        elif self.mode == "T":
            self.randomMoveAndLearn(chameleon)


    def followPolicy(self,state):
        #fist find the max util

        maxUtil = 0
        actionIndex = 0
        for i in range(9):
            [util,other] = self.brain[i].forward(state)
            if util > maxUtil:
                maxUtil = util
                actionIndex = i

        chosenMove = actionMapping(actionIndex)
                
        return chosenMove,actionIndex

    def randomMoveAndLearn(self,chameleon):
        smartCham = chameleon
        if not smartCham.waitingForReward:
            smartCham.firstReward = 0
            smartCham.states[0]=self.parent.checkState(smartCham)
            [positionChanges,smartCham.firstAction] = self.randomAction(smartCham.states[0])
            smartCham.smartMove(positionChanges)
            smartCham.states[1] = self.parent.checkState(smartCham)
            smartCham.firstReward = self.computeReward(smartCham.states[0],smartCham.states[1])
            smartCham.waitingForReward = True
            smartCham.waitingForLastReward = False
            
        else :
            [positionChanges,smartCham.secondAction] = self.randomAction(smartCham.states[1])
            smartCham.smartMove(positionChanges)
            smartCham.states[2]=self.parent.checkState(smartCham)
            smartCham.secondReward = self.computeReward(smartCham.states[1],smartCham.states[2])
            smartCham.waitingForLastReward = True


    def assignReward(self,chameleon,hasJustChanged=False):
        if self.mode == "R":
            return 0
        cham = chameleon
        reward = (cham.firstReward + cham.secondReward)

        if cham.waitingForLastReward or hasJustChanged :

            firstStateValue = 0
            for cell in cham.states[0][1:9]:
                firstStateValue += cell
                
            secondStateValue = 0
            for cell in cham.states[1][1:9]:
                secondStateValue += cell

            thirdStateValue = 0
            for cell in cham.states[2][1:9]:
                thirdStateValue += cell
                
            maxReward = maxRew[firstStateValue]
            if maxReward == 0:
                maxReward = 1

            if hasJustChanged :
                reward = -1
            elif reward >= 0:
                reward = float(reward)/maxReward
            elif reward <0:
                if maxPen[firstStateValue] == 0:
                    maxPen[firstStateValue] = 1
                reward = float(reward)/maxPen[firstStateValue]

            for i in range(3+int(3*abs(reward))):
                [o,y] = self.brain[cham.firstAction].forward(cham.states[0])
                if reward >= 0:
                    self.brain[cham.firstAction].backdrop(o,y,reward)
                else:
                    self.brain[cham.firstAction].backdrop(o,y,0)
                if i==0:
                    op = o
            
            if not hasJustChanged:
                cham.states[0]=cham.states[1]
                cham.states[1]=cham.states[2]
                cham.states[2] = []
                cham.firstAction = cham.secondAction
                cham.firstReward = cham.secondReward
            
            
    def randomAction(self,state):
        actionIndex = random.randint(0,8)
        [util, trainingValues] = self.brain[actionIndex].forward(state)
        chosenMove = actionMapping(actionIndex)
        return chosenMove,actionIndex

    def computeReward(self,stateX,stateY):
        stateX_value = 0
        stateY_value = 0
        for cell in stateX[1:9]:
            stateX_value += cell
        for cell in stateY[1:9]:
            stateY_value += cell

        reward = stateX_value - stateY_value
        if reward == 0 and stateX == 8:
            ans = 1
        elif reward == 0 and stateX == 0:
            ans = 1
        elif reward > maxRew[stateX_value]:
            ans = maxRew[stateX_value]
        elif reward < -maxPen[stateX_value]:
            ans = -maxPen[stateX_value]
        else :
            ans = reward
            
        return ans

        
        
        
        
        
        
            

    
