"""
    Copyright 2006,2007 Riccardo Taormina (den3b@hotmail.com)
    
    This file is part of SmartChameleons.

    SmartChameleons is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    SmartChameleons is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SmartChameleons; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

"""



from Tkinter import *
import tkMessageBox
import os

OutOfRangeError = "Out of range error"

class Dialog(Toplevel):
    def __init__(self, boss):

        Toplevel.__init__(self, boss)
        self.transient(boss)

        self.title("Enter Parameters")

        self.parent = boss

        self.result = None

        body = Frame(self)
        self.initial_focus = self.createBody(body)
        body.pack(padx=5, pady=5)

        self.createButtons()    #create the two buttons
        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)
        self.geometry("+%d+%d" % (boss.winfo_rootx()+100,
                                  boss.winfo_rooty()+100))
        self.initial_focus.focus_set()
        self.wait_window(self)

    def createBody(self, master):
        # creating the body of the message box

        Label(master, text="Horizontal length :").grid(row=0,sticky = 'W')
        Label(master, text="Vertical length :").grid(row=1,sticky = 'W')
        Label(master, text="Num. of cycles :").grid(row=2,sticky = 'W')
        Label(master, text="Num. of chameleons per color :").grid(row=3,sticky = 'W')
        Label(master, text="Random seed :").grid(row=4,sticky = 'W')

        self.e1 = Entry(master)
        self.e2 = Entry(master)
        self.e3 = Entry(master)
        self.e4 = Entry(master)
        self.e5 = Entry(master)
        
        self.e1.grid(row=0, column=1, sticky = W)
        self.e2.grid(row=1, column=1, sticky = W)
        self.e3.grid(row=2, column=1, sticky = W)
        self.e4.grid(row=3, column=1, sticky = W)
        self.e5.grid(row=4, column=1, sticky = W)

        self.e1.insert(0,"20")
        self.e2.insert(0,"20")
        self.e3.insert(0,"1000")
        self.e4.insert(0,"10")
        self.e5.insert(0,"12345")
        
        


    def createButtons(self):
      
        box = Frame(self)

        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack()

    def ok(self, event=None):
        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()
    
        self.apply()

        self.parent.panel.focus_set()
        self.destroy()

    def cancel(self, event=None):

        self.destroy()
        self.parent.destroy()


    def validate(self):
        try:
            maxX = int(self.e1.get())
            maxY = int(self.e2.get())
            n_cycles = int(self.e3.get())
            n_cham = int(self.e4.get())
            seed = int(self.e5.get())
            self.result = maxX,maxY,n_cycles,n_cham,seed
            if (maxX<=0 or maxX>30) or (maxY<=0 or maxY>20) or \
               (n_cycles<=0 or n_cycles>100000) or (n_cham<=0 or n_cham>(maxX*maxY/3)):
                raise OutOfRangeError            
            return 1        
        except ValueError:
            tkMessageBox.showwarning(
                "Wrong input!",
                "Values must be integer")
            
        except OutOfRangeError:
            text = "You must respect these constraints : \n\n"
            if (maxX<=0 or maxX>30) or (maxY<=0 or maxY>20):
                temp ="Horizontal length -> from 1 to 30\n" + "Vertical length -> from 1 to 15\n"
                text = text + temp
            elif n_cham<=0 or n_cham>(maxX*maxY/3):
                temp = "Num. of chameleons per each color -> from 1 to " + str(maxX*maxY/3)
                text = text + temp

            if (n_cycles<=0 or n_cycles>100000):
                temp = "Max num. of cycles -> from 1 to 100000\n"
                text = text + temp

            tkMessageBox.showwarning("Wrong input!!",text)
            
            return 0

    def apply(self):
        self.validate()
        self.parent.result=self.result

